@@ -211,8 +211,20 @@ impl ControlDir {
     }
 }
 
+impl std::fmt::Debug for ControlDir {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        f.write_fmt(format_args!("ControlDir({:?})", self.0))
+    }
+}
+
 pub struct ControlDirFormat(PyObject);
 
+impl ToPyObject for ControlDirFormat {
+    fn to_object(&self, py: Python) -> PyObject {
+        self.0.clone_ref(py)
+    }
+}
+
 impl Clone for ControlDirFormat {
     fn clone(&self) -> Self {
         Python::with_gil(|py| ControlDirFormat(self.0.clone_ref(py)))
@@ -237,6 +249,12 @@ impl Default for ControlDirFormat {
     }
 }
 
+#[test]
+fn test_control_dir_format_default() {
+    let d = ControlDirFormat::default();
+    d.get_format_string();
+}
+
 impl ControlDirFormat {
     pub fn get_format_string(&self) -> Vec<u8> {
         Python::with_gil(|py| {
@@ -283,7 +301,7 @@ impl From<PyErr> for OpenError {
 
         pyo3::Python::with_gil(|py| {
             if err.is_instance_of::<NotBranchError>(py) {
-                OpenError::NotFound(err.value(py).getattr("value").unwrap().extract().unwrap())
+                OpenError::NotFound(err.value(py).getattr("path").unwrap().extract().unwrap())
             } else if err.is_instance_of::<UnknownFormatError>(py) {
                 OpenError::UnknownFormat
             } else {
@@ -369,6 +387,23 @@ pub fn open_tree_or_branch(
     })
 }
 
+#[test]
+fn test_open_tree_or_branch() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    create_branch_convenience(&url::Url::from_directory_path(tmp_dir.path()).unwrap()).unwrap();
+    let (wt, branch) = open_tree_or_branch(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+        None,
+    )
+    .unwrap();
+    assert_eq!(wt.unwrap().basedir(), tmp_dir.path());
+    assert_eq!(
+        branch.get_user_url(),
+        url::Url::from_directory_path(tmp_dir.path()).unwrap()
+    );
+}
+
 pub fn open(
     url: impl AsLocation,
     possible_transports: Option<&mut Vec<Transport>>,
@@ -385,6 +420,36 @@ pub fn open(
     })
 }
 
+#[test]
+fn test_open() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+
+    let e = open(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap_err();
+
+    assert!(matches!(e, OpenError::NotFound(_)),);
+
+    let cd = create(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        "2a",
+        None,
+    )
+    .unwrap();
+
+    let od = open(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
+    assert_eq!(
+        cd.get_format().get_format_string(),
+        od.get_format().get_format_string()
+    );
+}
+
 pub fn create(
     url: impl AsLocation,
     format: impl AsFormat,
@@ -394,7 +459,9 @@ pub fn create(
         let m = py.import("breezy.controldir")?;
         let cd = m.getattr("ControlDir")?;
         let kwargs = PyDict::new(py);
-        kwargs.set_item("format", format.to_object(py))?;
+        if let Some(format) = format.as_format() {
+            kwargs.set_item("format", format.to_object(py))?;
+        }
         if let Some(possible_transports) = possible_transports {
             kwargs.set_item("possible_transports", possible_transports.to_object(py))?;
         }
@@ -403,6 +470,27 @@ pub fn create(
     })
 }
 
+#[test]
+fn test_create() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let cd = create(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        "2a",
+        None,
+    )
+    .unwrap();
+
+    let od = open(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
+    assert_eq!(
+        cd.get_format().get_format_string(),
+        od.get_format().get_format_string()
+    );
+}
+
 pub fn create_on_transport(
     transport: &Transport,
     format: impl AsFormat,
@@ -418,6 +506,17 @@ pub fn create_on_transport(
     })
 }
 
+#[test]
+fn test_create_on_transport() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let transport = crate::transport::get_transport(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
+    let _cd = create_on_transport(&transport, "2a").unwrap();
+}
+
 pub fn open_containing_from_transport(
     transport: &Transport,
     probers: Option<&[Prober]>,
@@ -441,6 +540,18 @@ pub fn open_containing_from_transport(
     })
 }
 
+#[test]
+fn test_open_containing_from_transport() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let transport = crate::transport::get_transport(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
+    let e = open_containing_from_transport(&transport, None).unwrap_err();
+    assert!(matches!(e, OpenError::NotFound(_)),);
+}
+
 pub fn open_from_transport(
     transport: &Transport,
     probers: Option<&[Prober]>,
@@ -461,7 +572,19 @@ pub fn open_from_transport(
     })
 }
 
-pub trait AsFormat: ToPyObject {
+#[test]
+fn test_open_from_transport() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let transport = crate::transport::get_transport(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
+    let e = open_from_transport(&transport, None).unwrap_err();
+    assert!(matches!(e, OpenError::NotFound(_)),);
+}
+
+pub trait AsFormat {
     fn as_format(&self) -> Option<ControlDirFormat>;
 }
 
@@ -469,19 +592,15 @@ impl AsFormat for &str {
     fn as_format(&self) -> Option<ControlDirFormat> {
         Python::with_gil(|py| {
             let m = py.import("breezy.controldir").ok()?;
-            let cd = m.getattr("ControlDirFormat").ok()?;
-            let format = cd.call_method1("get", (self.to_object(py),)).ok()?;
+            let cd = m.getattr("format_registry").ok()?;
+            let format = cd
+                .call_method1("make_controldir", (self.to_string(),))
+                .ok()?;
             Some(ControlDirFormat(format.to_object(py)))
         })
     }
 }
 
-impl ToPyObject for &ControlDirFormat {
-    fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
-    }
-}
-
 impl AsFormat for &ControlDirFormat {
     fn as_format(&self) -> Option<ControlDirFormat> {
         Some(ControlDirFormat(self.0.clone()))
@@ -497,6 +616,11 @@ pub fn create_branch_convenience(base: &url::Url) -> Result<Box<dyn Branch>, Cre
     })
 }
 
+/// Create a standalone working tree.
+///
+/// # Arguments
+/// * `base` - The base directory for the working tree.
+/// * `format` - The format of the working tree.
 pub fn create_standalone_workingtree(
     base: &std::path::Path,
     format: impl AsFormat,
@@ -505,8 +629,32 @@ pub fn create_standalone_workingtree(
     Python::with_gil(|py| {
         let m = py.import("breezy.controldir")?;
         let cd = m.getattr("ControlDir")?;
-        let format = format.to_object(py);
-        let wt = cd.call_method("create_standalone_workingtree", (base, format), None)?;
+        let format = format.as_format();
+        let wt = cd.call_method(
+            "create_standalone_workingtree",
+            (base, format.unwrap_or_default().to_object(py)),
+            None,
+        )?;
         Ok(WorkingTree(wt.to_object(py)))
     })
 }
+
+#[test]
+fn test_create_standalone_workingtree() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let wt = create_standalone_workingtree(tmp_dir.path(), "2a").unwrap();
+
+    assert_eq!(wt.basedir(), tmp_dir.path());
+}
+
+#[test]
+fn test_create_branch_convenience() {
+    let tmp_dir = tempfile::tempdir().unwrap();
+    let branch =
+        create_branch_convenience(&url::Url::from_directory_path(tmp_dir.path()).unwrap()).unwrap();
+
+    assert_eq!(
+        branch.get_user_url(),
+        url::Url::from_directory_path(tmp_dir.path()).unwrap()
+    );
+}
