@@ -17,6 +17,7 @@
 
 pub mod bazaar;
 pub mod branch;
+pub mod config;
 pub mod controldir;
 pub mod delta;
 pub mod diff;
@@ -83,10 +84,17 @@ impl std::fmt::Display for BreezyNotInstalled {
     }
 }
 
+#[cfg(feature = "auto-initialize")]
+#[ctor::ctor]
+fn ensure_initialized() {
+    init().unwrap();
+}
+
 static INIT_BREEZY: Once = Once::new();
 
 pub fn init() -> Result<(), BreezyNotInstalled> {
     INIT_BREEZY.call_once(|| {
+        pyo3::prepare_freethreaded_python();
         pyo3::Python::with_gil(|py| match py.import("breezy") {
             Ok(_) => Ok(()),
             Err(e) => {
@@ -111,13 +119,13 @@ pub fn init() -> Result<(), BreezyNotInstalled> {
             let f = m.getattr("ControlDirFormat").unwrap();
             f.call_method0("known_formats").unwrap();
         });
+
+        // Prevent race conditions
+        pyo3::Python::with_gil(|py| {
+            let m = py.import("breezy.config").unwrap();
+            m.call_method0("GlobalStack").unwrap();
+            m.call_method1("LocationStack", ("file:///",)).unwrap();
+        });
     });
     Ok(())
 }
-
-pub fn load_plugins() {
-    pyo3::Python::with_gil(|py| {
-        let m = py.import("breezy.plugin").unwrap();
-        m.call_method0("load_plugins").unwrap();
-    })
-}
