@@ -6,8 +6,10 @@ use pyo3::import_exception;
 use pyo3::prelude::*;
 
 import_exception!(breezy.commit, PointlessCommit);
+import_exception!(breezy.commit, NoWhoami);
 import_exception!(breezy.errors, NotBranchError);
 import_exception!(breezy.errors, DependencyNotPresent);
+import_exception!(breezy.errors, DivergedBranches);
 import_exception!(breezy.transport, NoSuchFile);
 
 pub type Path = std::path::Path;
@@ -165,6 +167,34 @@ impl From<Error> for PyErr {
     }
 }
 
+#[derive(Debug)]
+pub enum PullError {
+    DivergedBranches,
+    Other(PyErr),
+}
+
+impl From<PyErr> for PullError {
+    fn from(e: PyErr) -> Self {
+        Python::with_gil(|py| {
+            if e.is_instance_of::<DivergedBranches>(py) {
+                return PullError::DivergedBranches;
+            }
+            PullError::Other(e)
+        })
+    }
+}
+
+impl std::fmt::Display for PullError {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self {
+            PullError::DivergedBranches => write!(f, "Diverged branches"),
+            PullError::Other(e) => write!(f, "{}", e),
+        }
+    }
+}
+
+impl std::error::Error for PullError {}
+
 pub trait Tree: ToPyObject {
     fn get_tag_dict(&self) -> Result<std::collections::HashMap<String, RevisionId>, PyErr> {
         Python::with_gil(|py| {
@@ -472,6 +502,7 @@ impl Tree for RevisionTree {}
 #[derive(Debug)]
 pub enum CommitError {
     PointlessCommit,
+    NoWhoami,
     Other(PyErr),
 }
 
@@ -479,6 +510,7 @@ impl std::fmt::Display for CommitError {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
             CommitError::PointlessCommit => write!(f, "Pointless commit"),
+            CommitError::NoWhoami => write!(f, "No whoami"),
             CommitError::Other(e) => write!(f, "Other error: {}", e),
         }
     }
@@ -712,6 +744,8 @@ impl WorkingTree {
                 .map_err(|e| {
                     if e.is_instance_of::<PointlessCommit>(py) {
                         CommitError::PointlessCommit
+                    } else if e.is_instance_of::<NoWhoami>(py) {
+                        CommitError::NoWhoami
                     } else {
                         CommitError::Other(e)
                     }
@@ -728,10 +762,21 @@ impl WorkingTree {
         })
     }
 
-    pub fn pull(&self, source: &dyn crate::branch::Branch) -> Result<(), Error> {
+    pub fn pull(
+        &self,
+        source: &dyn crate::branch::Branch,
+        overwrite: Option<bool>,
+    ) -> Result<(), PullError> {
         Python::with_gil(|py| {
+            let kwargs = {
+                let kwargs = pyo3::types::PyDict::new(py);
+                if let Some(overwrite) = overwrite {
+                    kwargs.set_item("overwrite", overwrite).unwrap();
+                }
+                kwargs
+            };
             self.to_object(py)
-                .call_method1(py, "pull", (source.to_object(py),))
+                .call_method(py, "pull", (source.to_object(py),), Some(kwargs))
         })
         .map_err(|e| e.into())
         .map(|_| ())
